@@ -19,7 +19,7 @@ from aminer.parsing.WhiteSpaceLimitedDataModelElement import WhiteSpaceLimitedDa
 # This is a template for the "aminer" logfile miner tool. Copy
 # it to "config.py" and define your ruleset.
 
-config_properties = {}  # skipcq: PY-W0072
+config_properties = {}
 
 # Define the list of log resources to read from: the resources
 # named here do not need to exist when aminer is started. This
@@ -52,7 +52,7 @@ config_properties['AminerGroup'] = 'aminer'
 # to the 'AminerUser' but not group/world readable. On violation,
 # aminer will refuse to start. When undefined, '/var/lib/aminer'
 # is used.
-config_properties['Core.PersistenceDir'] = '/tmp/lib/aminer'  # skipcq: BAN-B108
+config_properties['Core.PersistenceDir'] = '/tmp/lib/aminer'
 
 # Define a target e-mail address to send alerts to. When undefined,
 # no e-mail notification hooks are added.
@@ -93,9 +93,10 @@ config_properties['LogPrefix'] = 'Original log line: '
 
 
 def build_analysis_pipeline(analysis_context):
-    """
-    Define the function to create pipeline for parsing the log data.
-    It has also to define an AtomizerFactory to instruct aminer how to process incoming data streams to create log atoms from them.
+    """Define the function to create pipeline for parsing the log data.
+
+    It has also to define an AtomizerFactory to instruct aminer how to
+    process incoming data streams to create log atoms from them.
     """
     date_format_string = b'%Y-%m-%d %H:%M:%S'
     cron = b' cron['
@@ -247,8 +248,8 @@ def build_analysis_pipeline(analysis_context):
 
     # Now define the AtomizerFactory using the model. A simple line based one is usually sufficient.
     from aminer.input.SimpleByteStreamLineAtomizerFactory import SimpleByteStreamLineAtomizerFactory
-    analysis_context.atomizer_factory = SimpleByteStreamLineAtomizerFactory(parsing_model, [simple_monotonic_timestamp_adjust],
-                                                                            anomaly_event_handlers)
+    analysis_context.atomizer_factory = SimpleByteStreamLineAtomizerFactory(
+        parsing_model, [simple_monotonic_timestamp_adjust], anomaly_event_handlers, use_real_time=True)
 
     # Just report all unparsed atoms to the event handlers.
     from aminer.analysis.UnparsedAtomHandlers import SimpleUnparsedAtomHandler, VerboseUnparsedAtomHandler
@@ -339,7 +340,8 @@ def build_analysis_pipeline(analysis_context):
     atom_filter.add_handler(new_match_path_detector)
 
     def tuple_transformation_function(match_value_list):
-        """Only allow output of the EnhancedNewMatchPathValueComboDetector after every 10th element."""
+        """Only allow output of the EnhancedNewMatchPathValueComboDetector
+        after every 10th element."""
         extra_data = enhanced_new_match_path_value_combo_detector.known_values_dict.get(tuple(match_value_list))
         if extra_data is not None:
             mod = 10
@@ -360,14 +362,14 @@ def build_analysis_pipeline(analysis_context):
     ip_match_action = Rules.EventGenerationMatchAction(
         "Analysis.Rules.IPv4InRFC1918MatchRule", "Private IP address occurred!", anomaly_event_handlers)
 
-    vdmt = Rules.ValueDependentModuloTimeMatchRule(None, 3, ["/model/ECD/j", "/model/ECD/k", "/model/ECD/l"], {b"e": [0, 2.95]}, [0, 3])
-    mt = Rules.ModuloTimeMatchRule(None, 3, 0, 3, None)
+    vdmt = Rules.ValueDependentModuloTimeMatchRule("vdmtmr", 3, ["/model/ECD/j", "/model/ECD/k", "/model/ECD/l"], {b"e": [0, 2.95]}, [0, 3])
+    mt = Rules.ModuloTimeMatchRule("mtmr", 3, 0, 3, None)
     time_allowlist_rules = [
         Rules.AndMatchRule([
             Rules.ParallelMatchRule([
                 Rules.ValueDependentDelegatedMatchRule([
                     '/model/ECD/g', '/model/ECD/h', '/model/ECD/i', '/model/ECD/j', '/model/ECD/k', '/model/ECD/l'], {
-                        (b"a",): mt, (b"b",): mt, (b"c",): mt, (b"d",): vdmt, (b"e",): vdmt, (b"f",): vdmt, None: mt}, mt),
+                        (b"a",): mt, (b"b",): mt, (b"c",): mt, (b"d",): vdmt, (b"e",): vdmt, (b"f",): vdmt}, mt),
                 Rules.IPv4InRFC1918MatchRule("/model/ParsingME/se2/IpAddressDataModelElement", ip_match_action),
                 Rules.DebugHistoryMatchRule(debug_match_result=True)
             ]),
@@ -453,7 +455,6 @@ def build_analysis_pipeline(analysis_context):
     rules = [Rules.PathExistsMatchRule('/model/CronAnnouncement/Run', a_class_selector),
              Rules.PathExistsMatchRule('/model/CronExecution/Job', b_class_selector)]
 
-    time_correlation_violation_detector = TimeCorrelationViolationDetector(analysis_context.aminer_config, rules, anomaly_event_handlers,
-                                                                           output_logline=True)
+    time_correlation_violation_detector = TimeCorrelationViolationDetector(analysis_context.aminer_config, rules, anomaly_event_handlers)
     analysis_context.register_component(time_correlation_violation_detector, component_name="TimeCorrelationViolationDetector")
     atom_filter.add_handler(time_correlation_violation_detector)
