@@ -3,11 +3,8 @@
 . ./testFunctions.sh
 
 LOGFILE=/tmp/syslog
-sudo mkdir /tmp/lib 2> /dev/null
-sudo mkdir /tmp/lib/aminer 2> /dev/null
-sudo chown -R $USER:$USER /tmp/lib/aminer 2> /dev/null
 sudo rm -r /tmp/lib/aminer/* 2> /dev/null
-sudo mkdir /tmp/lib/aminer/log 2> /dev/null
+sudo mkdir -p /tmp/lib/aminer/log
 sudo chown -R aminer:aminer /tmp/lib/aminer 2> /dev/null
 sudo rm $LOGFILE 2> /dev/null
 
