@@ -3,13 +3,13 @@
 source ../config
 
 # declare all expected values without the variable ones. These arrays are used to compare with the incoming log lines.
-declare -a NEW_PATH_HD_REPAIR_1=(" New path(es) detected" "NewMatchPathDetector: \"NewPath\" (1 lines)" "  /model/DiskUpgrade: " ": System rebooted for hard disk upgrade" "  /model/DiskUpgrade/DTM: " "  /model/DiskUpgrade/UNameSpace1: " "  /model/DiskUpgrade/UName: " "  /model/DiskUpgrade/UNameSpace2: " " /model/DiskUpgrade/User: " "  /model/DiskUpgrade/HDRepair:  System rebooted for hard disk upgrade" "['/model/DiskUpgrade', '/model/DiskUpgrade/DTM', '/model/DiskUpgrade/UNameSpace1', '/model/DiskUpgrade/UName', '/model/DiskUpgrade/UNameSpace2', '/model/DiskUpgrade/User', '/model/DiskUpgrade/HDRepair']" "Original log line: ")
-declare -a UNPARSED_ATOM_1=(" Unparsed atom received" "SimpleUnparsedAtomHandler: \"UnparsedHandler\" (1 lines)" " System rebooted for hard disk upgrad")
-declare -a UNPARSED_ATOM_2=(" Unparsed atom received" "SimpleUnparsedAtomHandler: \"UnparsedHandler\" (1 lines)" ": System rebooted for hard disk upgrade")
-declare -a NEW_PATH_HOME_PATH_ROOT_1=(" New path(es) detected" "NewMatchPathDetector: \"NewPath\" (1 lines)" "  /model/HomePath: The Path of the home directory shown by pwd of the user root is: /root" "  /model/HomePath/Pwd: The Path of the home directory shown by pwd of the user " "  /model/HomePath/Username: root" "  /model/HomePath/Is:  is: " "  /model/HomePath/Path: /root" "['/model/HomePath', '/model/HomePath/Pwd', '/model/HomePath/Username', '/model/HomePath/Is', '/model/HomePath/Path']" "Original log line: The Path of the home directory shown by pwd of the user root is: /root")
-declare -a NEW_VALUE_COMBINATION_HOME_PATH_ROOT_1=(" New value combination(s) detected" "NewMatchPathValueComboDetector: \"NewValueCombo\" (1 lines)" "(b'root', b'/root')" "Original log line: The Path of the home directory shown by pwd of the user root is: /root")
-declare -a NEW_VALUE_COMBINATION_HOME_PATH_USER_1=(" New value combination(s) detected" "NewMatchPathValueComboDetector: \"NewValueCombo\" (1 lines)" "(b'user', b'/home/user')" "Original log line: The Path of the home directory shown by pwd of the user user is: /home/user")
-declare -a NEW_VALUE_COMBINATION_HOME_PATH_GUEST_1=(" New value combination(s) detected" "NewMatchPathValueComboDetector: \"NewValueCombo\" (1 lines)" "(b'guest', b'/home/guest')" "Original log line: The Path of the home directory shown by pwd of the user guest is: /home/guest")
+declare -a NEW_PATH_HD_REPAIR_1=("New path(s) detected" "NewMatchPathDetector: \"NewPath\" (1 lines)" "  /model/DiskUpgrade: " ": System rebooted for hard disk upgrade" "  /model/DiskUpgrade/DTM: " "  /model/DiskUpgrade/UNameSpace1: " "  /model/DiskUpgrade/UName: " "  /model/DiskUpgrade/UNameSpace2: " " /model/DiskUpgrade/User: " "  /model/DiskUpgrade/HDRepair:  System rebooted for hard disk upgrade" "['/model/DiskUpgrade', '/model/DiskUpgrade/DTM', '/model/DiskUpgrade/UNameSpace1', '/model/DiskUpgrade/UName', '/model/DiskUpgrade/UNameSpace2', '/model/DiskUpgrade/User', '/model/DiskUpgrade/HDRepair']" "Original log line: ")
+declare -a UNPARSED_ATOM_1=("Unparsed atom received" "SimpleUnparsedAtomHandler: \"UnparsedHandler\" (1 lines)" " System rebooted for hard disk upgrad")
+declare -a UNPARSED_ATOM_2=("Unparsed atom received" "SimpleUnparsedAtomHandler: \"UnparsedHandler\" (1 lines)" ": System rebooted for hard disk upgrade")
+declare -a NEW_PATH_HOME_PATH_ROOT_1=("New path(s) detected" "NewMatchPathDetector: \"NewPath\" (1 lines)" "  /model/HomePath: The Path of the home directory shown by pwd of the user root is: /root" "  /model/HomePath/Pwd: The Path of the home directory shown by pwd of the user " "  /model/HomePath/Username: root" "  /model/HomePath/Is:  is: " "  /model/HomePath/Path: /root" "['/model/HomePath', '/model/HomePath/Pwd', '/model/HomePath/Username', '/model/HomePath/Is', '/model/HomePath/Path']" "Original log line: The Path of the home directory shown by pwd of the user root is: /root")
+declare -a NEW_VALUE_COMBINATION_HOME_PATH_ROOT_1=("New value combination(s) detected" "NewMatchPathValueComboDetector: \"NewValueCombo\" (1 lines)" "(b'root', b'/root')" "Original log line: The Path of the home directory shown by pwd of the user root is: /root")
+declare -a NEW_VALUE_COMBINATION_HOME_PATH_USER_1=("New value combination(s) detected" "NewMatchPathValueComboDetector: \"NewValueCombo\" (1 lines)" "(b'user', b'/home/user')" "Original log line: The Path of the home directory shown by pwd of the user user is: /home/user")
+declare -a NEW_VALUE_COMBINATION_HOME_PATH_GUEST_1=("New value combination(s) detected" "NewMatchPathValueComboDetector: \"NewValueCombo\" (1 lines)" "(b'guest', b'/home/guest')" "Original log line: The Path of the home directory shown by pwd of the user guest is: /home/guest")
 declare -a JSON_OUTPUT=()
 read -r -d '' VAR << END
   {
@@ -38,7 +38,8 @@ read -r -d '' VAR << END
     "AnalysisComponentType": "SimpleUnparsedAtomHandler",
     "AnalysisComponentName": "UnparsedHandler",
     "Message": "Unparsed atom received",
-    "PersistenceFileName": null
+    "PersistenceFileName": null,
+    "LogResource": "file:///tmp/syslog"
   }
 }
 {
@@ -46,7 +47,7 @@ read -r -d '' VAR << END
     "AnalysisComponentIdentifier": 1,
     "AnalysisComponentType": "NewMatchPathDetector",
     "AnalysisComponentName": "NewPath",
-    "Message": "New path(es) detected",
+    "Message": "New path(s) detected",
     "PersistenceFileName": "Default",
     "TrainingMode": true,
     "AffectedLogAtomPaths": [
@@ -57,7 +58,8 @@ read -r -d '' VAR << END
       "/model/DiskUpgrade/UNameSpace2",
       "/model/DiskUpgrade/User",
       "/model/DiskUpgrade/HDRepair"
-    ]
+    ],
+    "LogResource": "file:///tmp/auth.log"
   },
   "LogData": {
     "RawLogData": [
@@ -122,7 +124,8 @@ read -r -d '' VAR << END
     "AnalysisComponentType": "SimpleUnparsedAtomHandler",
     "AnalysisComponentName": "UnparsedHandler",
     "Message": "Unparsed atom received",
-    "PersistenceFileName": null
+    "PersistenceFileName": null,
+    "LogResource": "file:///tmp/auth.log"
   }
 }
 {
@@ -151,7 +154,8 @@ read -r -d '' VAR << END
     "AnalysisComponentType": "SimpleUnparsedAtomHandler",
     "AnalysisComponentName": "UnparsedHandler",
     "Message": "Unparsed atom received",
-    "PersistenceFileName": null
+    "PersistenceFileName": null,
+    "LogResource": "file:///tmp/syslog"
   }
 }
 {
@@ -159,7 +163,7 @@ read -r -d '' VAR << END
     "AnalysisComponentIdentifier": 1,
     "AnalysisComponentType": "NewMatchPathDetector",
     "AnalysisComponentName": "NewPath",
-    "Message": "New path(es) detected",
+    "Message": "New path(s) detected",
     "PersistenceFileName": "Default",
     "TrainingMode": true,
     "AffectedLogAtomPaths": [
@@ -168,7 +172,8 @@ read -r -d '' VAR << END
       "/model/HomePath/Username",
       "/model/HomePath/Is",
       "/model/HomePath/Path"
-    ]
+    ],
+    "LogResource": "file:///tmp/auth.log"
   },
   "LogData": {
     "RawLogData": [
@@ -209,7 +214,8 @@ read -r -d '' VAR << END
     "AffectedLogAtomValues": [
       "root",
       "/root"
-    ]
+    ],
+    "LogResource": "file:///tmp/auth.log"
   },
   "LogData": {
     "RawLogData": [
@@ -250,7 +256,8 @@ read -r -d '' VAR << END
     "AffectedLogAtomValues": [
       "user",
       "/home/user"
-    ]
+    ],
+    "LogResource": "file:///tmp/syslog"
   },
   "LogData": {
     "RawLogData": [
@@ -291,7 +298,8 @@ read -r -d '' VAR << END
     "AffectedLogAtomValues": [
       "guest",
       "/home/guest"
-    ]
+    ],
+    "LogResource": "file:///tmp/auth.log"
   },
   "LogData": {
     "RawLogData": [
@@ -319,6 +327,40 @@ read -r -d '' VAR << END
 }
 END
 JSON_OUTPUT+=("$VAR")
+read -r -d '' VAR << END
+{
+  "AnalysisComponent": {
+    "AnalysisComponentIdentifier": 3,
+    "AnalysisComponentType": "SlidingEventFrequencyDetector",
+    "AnalysisComponentName": "SlidingEventFrequencyDetector",
+    "Message": "Frequency exceeds range for the first time",
+    "PersistenceFileName": "Default",
+    "TrainingMode": true,
+    "AffectedLogAtomPaths": [],
+    "AffectedLogAtomValues": [
+      "/model/HomePath",
+      "/model/HomePath/Pwd",
+      "/model/HomePath/Username",
+      "/model/HomePath/Is",
+      "/model/HomePath/Path"
+    ],
+    "LogResource": "file:///tmp/auth.log"
+  },
+  "FrequencyData": {
+    "ExpectedLogAtomValuesFrequencyRange": [
+      0,
+      1
+    ],
+    "LogAtomValuesFrequency": 2,
+    "WindowSize": 2
+  },
+  "LogData": {
+    "RawLogData": [
+      "The Path of the home directory shown by pwd of the user guest is: /home/guest"
+    ],
+    "Timestamps": [
+END
+JSON_OUTPUT+=("$VAR")
 
 # These strings are used in the isExpectedOutput()-function to identify the next array to be compared with.
 NEW_PATH_HD_REPAIR="new_path_hd_repair"
@@ -449,7 +491,7 @@ function isExpectedOutput() {
 		# At the end of an paragraph stop reading the file and go to return from the function.
   		if [ "$line" == "" ]; then
 			break
-		# When the current line contains the expected value at the expected position, 
+		# When the current line contains the expected value at the expected position,
 		# read until the following values do not match or the EXPECTED array ends.
 		elif [[ "$line" == *"${EXPECTED[$i - $before - $temp]}"* ]]; then
 			i=$((i + 1))
@@ -579,7 +621,7 @@ function checkAllMails() {
 	echo ""
 
 	i=1
-	while [ $i -lt $linecount ] 
+	while [ $i -lt $linecount ]
 	do
 		sudo echo p | mail > /tmp/out
 		input="/tmp/out"
@@ -619,7 +661,7 @@ function checkAllMails() {
 				i=$(($i-1))
 				break
 			fi
-			# If the time is lesser than the start time of the integration test, an old mail is found. 
+			# If the time is lesser than the start time of the integration test, an old mail is found.
 			if [[ "$searched" == *"Date: "* ]]; then
 				d="${searched:6}"
 				dat=`date -d "$d" +%s`
@@ -639,7 +681,7 @@ function checkAllMails() {
 		done < "$input"
 		i=$(($i+1))
 	done
-	
+
 	echo "finished waiting.."
 	return $res
 }
@@ -647,12 +689,12 @@ function checkAllMails() {
 # This function checks if the output of the Syslog is as expected.
 function checkAllSyslogs(){
 	sudo tail -n 1000 /var/log/syslog > /tmp/out
-	
+
 	lastLine=`tail -n 1 /tmp/output`
 	if [[ $lastLine == "" ]]; then
 		sudo sed -i "$ d" /tmp/output
 	fi
-	
+
 	cntr=0
 	input="/tmp/out"
 	i=0
@@ -663,7 +705,7 @@ function checkAllSyslogs(){
 		d="${searched:0:15}"
 		dat=`date -d "$d" +%s`
 		# Ignore all old syslogs and just process the current ones.
-		if [[ !($dat -lt $time) ]]; then			
+		if [[ !($dat -lt $time) ]]; then
 			expected="/tmp/output"
 			found=false
 			g=0
@@ -699,7 +741,7 @@ function checkAllSyslogs(){
 		fi
 	done < "$input"
 	echo "finished waiting.."
-	# $NUMBER_OF_LOG_LINES must always be the number of paragraphs in /tmp/output minus one, 
+	# $NUMBER_OF_LOG_LINES must always be the number of paragraphs in /tmp/output minus one,
 	# as there is no empty line before the first paragraph.
 	if [ $i == $NUMBER_OF_LOG_LINES ]; then
 		return 0
@@ -709,7 +751,24 @@ function checkAllSyslogs(){
 
 # This function checks if the output of the Kafka Topic is as expected.
 function checkKafkaTopic(){
-  out=$($KAFKA_VERSIONSTRING/bin/kafka-console-consumer.sh --bootstrap-server localhost:9092 --topic test_topic --from-beginning --timeout-ms 3000)
+  out=$($KAFKA_VERSIONSTRING/bin/kafka-console-consumer.sh --bootstrap-server localhost:9092 --topic test_topic --from-beginning --timeout-ms 60000)
+  for t in "${JSON_OUTPUT[@]}"
+  do
+    if [[ $out != *"$t"* ]]; then
+      echo "searched: $t"
+      echo
+      echo "remaining output: $out"
+      return 1
+    fi
+    # cut the output string to remove timestamps and datetimes.
+    out=${out#*$t}
+  done
+  return 0
+}
+
+# This function checks if the output of the ZMQ Topic is as expected.
+function checkZmqTopic(){
+  out=$(cat /tmp/zmq)
   for t in "${JSON_OUTPUT[@]}"
   do
     if [[ $out != *"$t"* ]]; then
