@@ -1,3 +1,5 @@
+#!/bin/bash
+
 echo localhost | sudo tee /etc/hostname > /dev/null
 cd integration
 script=$1
@@ -12,6 +14,8 @@ do
 	cntr=$(($cntr+1))
 done
 
+cp zmq_subscriber.py /tmp
+
 sudo ./$script
 exit_code=$?
 
@@ -28,6 +32,7 @@ test -e /var/mail/mail && sudo rm -f /var/mail/mail
 cd ..
 sudo rm /tmp/syslog
 sudo rm /tmp/output
+sudo rm /tmp/zmq_subscriber.py
 test -e /tmp/out && sudo rm /tmp/out
 test -e /tmp/auth.log && sudo rm /tmp/auth.log
 exit $exit_code
