@@ -1,6 +1,6 @@
 from time import sleep
 import socket
-sock_name = '/tmp/test5unixSocket.sock'  # skipcq: BAN-B108
+sock_name = '/tmp/test5unixSocket.sock'
 
 sleep(0.5)
 client = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
