@@ -11,17 +11,7 @@ class DecimalIntegerValueModelElementTest(TestBase):
     id_ = "integer"
     path = "path"
 
-    def test1get_id(self):
-        """Test if get_id works properly."""
-        decimal_integer_me = DecimalIntegerValueModelElement(self.id_)
-        self.assertEqual(decimal_integer_me.get_id(), self.id_)
-
-    def test2get_child_elements(self):
-        """Test if get_child_elements returns None."""
-        decimal_integer_me = DecimalIntegerValueModelElement(self.id_)
-        self.assertEqual(decimal_integer_me.get_child_elements(), None)
-
-    def test3get_match_element_default_values(self):
+    def test1get_match_element_default_values(self):
         """Test valid integer values with default values of value_sign_type and value_pad_type."""
         decimal_integer_value_me = DecimalIntegerValueModelElement(
             self.id_, DecimalIntegerValueModelElement.SIGN_TYPE_NONE, DecimalIntegerValueModelElement.PAD_TYPE_NONE)
@@ -61,7 +51,7 @@ class DecimalIntegerValueModelElementTest(TestBase):
         match_element = decimal_integer_value_me.get_match_element(self.path, match_context)
         self.compare_match_results(data, match_element, match_context, self.id_, self.path, value, 0, None)
 
-    def test4get_match_element_default_values_no_match(self):
+    def test2get_match_element_default_values_no_match(self):
         """Test not matching values with default values of value_sign_type and value_pad_type."""
         decimal_integer_value_me = DecimalIntegerValueModelElement(
             self.id_, DecimalIntegerValueModelElement.SIGN_TYPE_NONE, DecimalIntegerValueModelElement.PAD_TYPE_NONE)
@@ -133,7 +123,7 @@ class DecimalIntegerValueModelElementTest(TestBase):
         match_element = decimal_integer_value_me.get_match_element(self.path, match_context)
         self.compare_no_match_results(data, match_element, match_context)
 
-    def test5get_match_element_optional_zero_values(self):
+    def test3get_match_element_optional_zero_values(self):
         """Test valid float values with "optional" or "zero" values of value_sign_type and value_pad_type."""
         decimal_integer_value_me = DecimalIntegerValueModelElement(
             self.id_, DecimalIntegerValueModelElement.SIGN_TYPE_OPTIONAL, DecimalIntegerValueModelElement.PAD_TYPE_ZERO)
@@ -215,7 +205,7 @@ class DecimalIntegerValueModelElementTest(TestBase):
         match_element = decimal_integer_value_me.get_match_element(self.path, match_context)
         self.compare_match_results(data, match_element, match_context, self.id_, self.path, value, 0, None)
 
-    def test6get_match_element_optional_zero_values_no_match(self):
+    def test4get_match_element_optional_zero_values_no_match(self):
         """Test not matching values with default values of value_sign_type and value_pad_type."""
         decimal_integer_value_me = DecimalIntegerValueModelElement(
             self.id_, DecimalIntegerValueModelElement.SIGN_TYPE_OPTIONAL, DecimalIntegerValueModelElement.PAD_TYPE_ZERO)
@@ -255,7 +245,7 @@ class DecimalIntegerValueModelElementTest(TestBase):
         match_element = decimal_integer_value_me.get_match_element(self.path, match_context)
         self.compare_no_match_results(data, match_element, match_context)
 
-    def test7get_match_element_mandatory_blank_values(self):
+    def test5get_match_element_mandatory_blank_values(self):
         """Test valid float values with "mandatory" or "blank" values of value_sign_type and value_pad_type."""
         decimal_integer_value_me = DecimalIntegerValueModelElement(
             self.id_, DecimalIntegerValueModelElement.SIGN_TYPE_MANDATORY, DecimalIntegerValueModelElement.PAD_TYPE_BLANK)
@@ -325,7 +315,7 @@ class DecimalIntegerValueModelElementTest(TestBase):
         match_element = decimal_integer_value_me.get_match_element(self.path, match_context)
         self.compare_match_results(data, match_element, match_context, self.id_, self.path, value, 0, None)
 
-    def test8get_match_element_mandatory_blank_values_no_match(self):
+    def test6get_match_element_mandatory_blank_values_no_match(self):
         """Test not matching values with default values of value_sign_type and value_pad_type."""
         decimal_integer_value_me = DecimalIntegerValueModelElement(
             self.id_, DecimalIntegerValueModelElement.SIGN_TYPE_MANDATORY, DecimalIntegerValueModelElement.PAD_TYPE_BLANK)
@@ -400,7 +390,7 @@ class DecimalIntegerValueModelElementTest(TestBase):
         match_element = decimal_integer_value_me.get_match_element(self.path, match_context)
         self.compare_no_match_results(data, match_element, match_context)
 
-    def test9element_id_input_validation(self):
+    def test7element_id_input_validation(self):
         """Check if element_id is validated."""
         self.assertRaises(ValueError, DecimalIntegerValueModelElement, "")  # empty element_id
         self.assertRaises(TypeError, DecimalIntegerValueModelElement, None)  # None element_id
@@ -414,7 +404,7 @@ class DecimalIntegerValueModelElementTest(TestBase):
         self.assertRaises(TypeError, DecimalIntegerValueModelElement, ())  # empty tuple element_id is not allowed
         self.assertRaises(TypeError, DecimalIntegerValueModelElement, set())  # empty set element_id is not allowed
 
-    def test10value_sign_type_input_validation(self):
+    def test9value_sign_type_input_validation(self):
         """Check if value_sign_type is validated."""
         DecimalIntegerValueModelElement(self.id_, value_sign_type="none")
         DecimalIntegerValueModelElement(self.id_, value_sign_type="optional")
@@ -432,7 +422,7 @@ class DecimalIntegerValueModelElementTest(TestBase):
         self.assertRaises(TypeError, DecimalIntegerValueModelElement, self.id_, value_sign_type=())
         self.assertRaises(TypeError, DecimalIntegerValueModelElement, self.id_, value_sign_type=set())
 
-    def test11value_pad_type_input_validation(self):
+    def test10value_pad_type_input_validation(self):
         """Check if value_pad_type is validated."""
         DecimalIntegerValueModelElement(self.id_, value_pad_type="none")
         DecimalIntegerValueModelElement(self.id_, value_pad_type="zero")
@@ -450,7 +440,7 @@ class DecimalIntegerValueModelElementTest(TestBase):
         self.assertRaises(TypeError, DecimalIntegerValueModelElement, self.id_, value_pad_type=())
         self.assertRaises(TypeError, DecimalIntegerValueModelElement, self.id_, value_pad_type=set())
 
-    def test12get_match_element_match_context_input_validation(self):
+    def test11get_match_element_match_context_input_validation(self):
         """Check if an exception is raised, when other classes than MatchContext are used in get_match_element."""
         model_element = DecimalIntegerValueModelElement(self.id_)
         data = b"123.22"
