@@ -13,18 +13,7 @@ class RepeatedElementDataModelElementTest(TestBase):
     fixed_id = "fixed"
     fixed_data = b"fixed data "
 
-    def test1get_id(self):
-        """Test if get_id works properly."""
-        repeated_dme = RepeatedElementDataModelElement(self.id_, DummyFixedDataModelElement(self.fixed_id, self.fixed_data))
-        self.assertEqual(repeated_dme.get_id(), self.id_)
-
-    def test2get_child_elements(self):
-        """Test if get_child_elements returns None."""
-        fixed_dme = DummyFixedDataModelElement(self.fixed_id, self.fixed_data)
-        repeated_dme = RepeatedElementDataModelElement(self.id_, fixed_dme)
-        self.assertEqual(repeated_dme.get_child_elements(), [fixed_dme])
-
-    def test3get_match_element_valid_match(self):
+    def test1get_match_element_valid_match(self):
         """Parse matching substring from MatchContext and check if the MatchContext was updated with all characters."""
         fixed_dme = DummyFixedDataModelElement(self.fixed_id, self.fixed_data)
         repeated_dme = RepeatedElementDataModelElement(self.id_, DummyFixedDataModelElement(self.fixed_id, self.fixed_data))
@@ -54,7 +43,7 @@ class RepeatedElementDataModelElementTest(TestBase):
             fixed_dme.get_match_element("%s/%s/1" % (self.path, self.id_), DummyMatchContext(data))
         ])
 
-    def test4get_match_element_min_max_repeats(self):
+    def test2get_match_element_min_max_repeats(self):
         """This test case verifies the functionality of setting the minimal and maximal repeats."""
         fixed_dme = DummyFixedDataModelElement(self.fixed_id, self.fixed_data)
         repeated_dme = RepeatedElementDataModelElement(self.id_, fixed_dme, min_repeat=2, max_repeat=5)
@@ -138,7 +127,7 @@ class RepeatedElementDataModelElementTest(TestBase):
         match_element = same_min_max_repeat_dme.get_match_element(self.path, match_context)
         self.compare_no_match_results(data, match_element, match_context)
 
-    def test5element_id_input_validation(self):
+    def test3element_id_input_validation(self):
         """Check if element_id is validated."""
         fixed_dme = DummyFixedDataModelElement(self.fixed_id, self.fixed_data)
         self.assertRaises(ValueError, RepeatedElementDataModelElement, "", fixed_dme)
@@ -153,7 +142,7 @@ class RepeatedElementDataModelElementTest(TestBase):
         self.assertRaises(TypeError, RepeatedElementDataModelElement, (), fixed_dme)
         self.assertRaises(TypeError, RepeatedElementDataModelElement, set(), fixed_dme)
 
-    def test6repeated_element_input_validation(self):
+    def test4repeated_element_input_validation(self):
         """Check if repeated_element is validated."""
         self.assertRaises(TypeError, RepeatedElementDataModelElement, self.id_, "string")
         self.assertRaises(TypeError, RepeatedElementDataModelElement, self.id_, None)
@@ -167,7 +156,7 @@ class RepeatedElementDataModelElementTest(TestBase):
         self.assertRaises(TypeError, RepeatedElementDataModelElement, self.id_, ())
         self.assertRaises(TypeError, RepeatedElementDataModelElement, self.id_, set())
 
-    def test7min_repeat_input_validation(self):
+    def test5min_repeat_input_validation(self):
         """Check if min_repeat is validated."""
         fixed_dme = DummyFixedDataModelElement(self.fixed_id, self.fixed_data)
         self.assertRaises(TypeError, RepeatedElementDataModelElement, self.id_, fixed_dme, min_repeat="string")
@@ -182,7 +171,7 @@ class RepeatedElementDataModelElementTest(TestBase):
         self.assertRaises(TypeError, RepeatedElementDataModelElement, self.id_, fixed_dme, min_repeat=())
         self.assertRaises(TypeError, RepeatedElementDataModelElement, self.id_, fixed_dme, min_repeat=set())
 
-    def test8max_repeat_input_validation(self):
+    def test6max_repeat_input_validation(self):
         """Check if max_repeat is validated."""
         fixed_dme = DummyFixedDataModelElement(self.fixed_id, self.fixed_data)
         self.assertRaises(TypeError, RepeatedElementDataModelElement, self.id_, fixed_dme, max_repeat="string")
@@ -198,7 +187,7 @@ class RepeatedElementDataModelElementTest(TestBase):
         self.assertRaises(TypeError, RepeatedElementDataModelElement, self.id_, fixed_dme, max_repeat=())
         self.assertRaises(TypeError, RepeatedElementDataModelElement, self.id_, fixed_dme, max_repeat=set())
 
-    def test9get_match_element_match_context_input_validation(self):
+    def test7get_match_element_match_context_input_validation(self):
         """Check if an exception is raised, when other classes than MatchContext are used in get_match_element."""
         model_element = RepeatedElementDataModelElement(self.id_, DummyFixedDataModelElement(self.fixed_id, self.fixed_data))
         data = b"fixed data"
