@@ -11,17 +11,7 @@ class WhiteSpaceLimitedDataModelElementTest(TestBase):
     id_ = "whitespace"
     path = "path"
 
-    def test1get_id(self):
-        """Test if get_id works properly."""
-        whitespace_dme = WhiteSpaceLimitedDataModelElement(self.id_)
-        self.assertEqual(whitespace_dme.get_id(), self.id_)
-
-    def test2get_child_elements(self):
-        """Test if get_child_elements returns None."""
-        whitespace_dme = WhiteSpaceLimitedDataModelElement(self.id_)
-        self.assertEqual(whitespace_dme.get_child_elements(), None)
-
-    def test3get_match_element_valid_match(self):
+    def test1get_match_element_valid_match(self):
         """Parse matching substring from MatchContext and check if the MatchContext was updated with all characters."""
         data = b"space: ,tab:\t"
         value = b"space:"
@@ -71,7 +61,7 @@ class WhiteSpaceLimitedDataModelElementTest(TestBase):
         match_element = whitespace_dme.get_match_element(self.path, match_context)
         self.compare_match_results(data, match_element, match_context, self.id_, self.path, value, value, None)
 
-    def test4get_match_element_no_match(self):
+    def test2get_match_element_no_match(self):
         """Parse not matching substring from MatchContext and check if the MatchContext was not changed."""
         data = b""
         match_context = DummyMatchContext(data)
@@ -91,7 +81,7 @@ class WhiteSpaceLimitedDataModelElementTest(TestBase):
         match_element = whitespace_dme.get_match_element(self.path, match_context)
         self.compare_no_match_results(data, match_element, match_context)
 
-    def test5element_id_input_validation(self):
+    def test3element_id_input_validation(self):
         """Check if element_id is validated."""
         self.assertRaises(ValueError, WhiteSpaceLimitedDataModelElement, "")
         self.assertRaises(TypeError, WhiteSpaceLimitedDataModelElement, None)
@@ -105,7 +95,7 @@ class WhiteSpaceLimitedDataModelElementTest(TestBase):
         self.assertRaises(TypeError, WhiteSpaceLimitedDataModelElement, ())
         self.assertRaises(TypeError, WhiteSpaceLimitedDataModelElement, set())
 
-    def test6get_match_element_match_context_input_validation(self):
+    def test4get_match_element_match_context_input_validation(self):
         """Check if an exception is raised, when other classes than MatchContext are used in get_match_element."""
         model_element = WhiteSpaceLimitedDataModelElement(self.id_)
         data = b"space: ,tab:\t"
