@@ -24,6 +24,10 @@ def get_model():
                 FixedDataModelElement("start", b"Start queue run: pid="),
                 DecimalIntegerValueModelElement("pid"),
                 ]),
+            SequenceModelElement("start", [
+                FixedDataModelElement("start", b"TLS error"),
+                AnyByteDataModelElement('remainder')
+                ]),
             SequenceModelElement("end", [
                 FixedDataModelElement("end", b"End queue run: pid="),
                 DecimalIntegerValueModelElement("pid"),
@@ -32,6 +36,12 @@ def get_model():
                 FixedDataModelElement("no_host_found_str", b"no host name found for IP address "),
                 IpAddressDataModelElement("ip"),
                 ]),
+            SequenceModelElement("start_daemon", [
+                FixedDataModelElement("start_daemon_str", b"exim "),
+                DelimitedDataModelElement("version", b" "),
+                FixedDataModelElement("start_deamon_str2", b" daemon started"),
+                AnyByteDataModelElement("msg")
+                ]),
             SequenceModelElement("vrfy_failed", [
                 FixedDataModelElement("vrfy_failed_str", b"VRFY failed for "),
                 DelimitedDataModelElement("mail", b" "),
@@ -41,6 +51,10 @@ def get_model():
                 IpAddressDataModelElement("ip"),
                 FixedDataModelElement("sp2", b"]")
                 ]),
+            SequenceModelElement("end", [
+                DelimitedDataModelElement("spool", b" "),
+                FixedDataModelElement("spool_file_locked", b" Spool file is locked (another process is handling this message)")
+                ]),
             SequenceModelElement("mail", [
                 DelimitedDataModelElement("id", b" "),
                 FirstMatchModelElement("dir", [
@@ -83,30 +97,65 @@ def get_model():
                                         AnyByteDataModelElement("id")
                                         ])
                                     )
-                                ])
+                                ]),
+                            AnyByteDataModelElement('remainder')
                             ])
                         ]),
                     SequenceModelElement("dir_out", [
                         FixedDataModelElement("in", b" => "),
                         DelimitedDataModelElement("name", b" "),
-                        FixedDataModelElement("sp1", b" "),
-                        OptionalMatchModelElement(
-                            "mail_opt", SequenceModelElement("mail", [
-                                FixedDataModelElement("brack1", b"("),
-                                DelimitedDataModelElement("brack_mail", b")"),
-                                FixedDataModelElement("brack2", b") "),
-                                ])),
-                        FixedDataModelElement("sp2", b"<"),
-                        DelimitedDataModelElement("mail", b">"),
-                        FixedDataModelElement("r_str", b"> R="),
-                        DelimitedDataModelElement("r", b" "),
-                        FixedDataModelElement("t_str", b" T="),
-                        VariableByteDataModelElement("t", alphabet),
+                        FirstMatchModelElement('fm', [
+                            SequenceModelElement('seq', [
+                                FixedDataModelElement("sp1", b" "),
+                                OptionalMatchModelElement(
+                                    "mail_opt",
+                                    SequenceModelElement(
+                                        "mail", [
+                                            FixedDataModelElement("brack1", b"("),
+                                            DelimitedDataModelElement("brack_mail", b")"),
+                                            FixedDataModelElement("brack2", b") "),
+                                        ])
+                                ),
+                                OptionalMatchModelElement(
+                                    "opt",
+                                    SequenceModelElement(
+                                        "seq", [
+                                            FixedDataModelElement("sp2", b"<"),
+                                            DelimitedDataModelElement("mail", b">"),
+                                            FixedDataModelElement("closing_brack", b"> "),
+                                        ])
+                                ),
+                                FixedDataModelElement("r_str", b"R="),
+                                DelimitedDataModelElement("r", b" "),
+                                FixedDataModelElement("t_str", b" T="),
+                                VariableByteDataModelElement("t", alphabet),
+                                OptionalMatchModelElement(
+                                    "param_opt",
+                                    SequenceModelElement(
+                                        "seq", [
+                                            FixedDataModelElement("h_str", b" H="),
+                                            DelimitedDataModelElement("h", b" X="),
+                                            FixedDataModelElement("x_str", b" X="),
+                                            DelimitedDataModelElement("x", b" CV="),
+                                            FixedDataModelElement("cv_str", b" CV="),
+                                            DelimitedDataModelElement("cv", b" DN="),
+                                            FixedDataModelElement("dn_str", b" DN="),
+                                            DelimitedDataModelElement("dn", b" C="),
+                                            AnyByteDataModelElement("c"),
+                                        ]))
+                                ]),
+                            ])
                         ]),
                     SequenceModelElement("aster", [
                         FixedDataModelElement("aster", b" ** "),
                         DelimitedDataModelElement("command", b" "),
                         FixedDataModelElement("headers_str", b' Too many "Received" headers - suspected mail loop')]),
+                    SequenceModelElement("prdr", [
+                        FixedDataModelElement("prdr", b" PRDR "),
+                        AnyByteDataModelElement('remainder')]),
+                    SequenceModelElement("arrw", [
+                        FixedDataModelElement("arrw", b" -> "),
+                        AnyByteDataModelElement('remainder')]),
                     FixedDataModelElement("completed", b" Completed"),
                     FixedDataModelElement("frozen", b" Message is frozen"),
                     FixedDataModelElement("frozen", b" Frozen (delivery error message)")
