@@ -7,8 +7,9 @@ from aminer.parsing.VariableByteDataModelElement import VariableByteDataModelEle
 
 
 def get_model(time_model=None):
-    """
-    Return the model for parsing a standard syslog preamble including timestamp and hostname.
+    """Return the model for parsing a standard syslog preamble including
+    timestamp and hostname.
+
     @param time_model when not none, the given model element is used for parsing timestamps. Otherwise a standard DateTimeModelElement
     with format b"%b %d %H:%M:%S" is created. CAVEAT: the standard model may not work when log data timestamp locale does not match
     host or shell environment locale. See MultiLocaleDatetime_modelElement instead.
