@@ -1,16 +1,15 @@
-#!/usr/bin/python3 -BbbEIsSttW all
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
-
-"""
-This is the main program of the "aminer" logfile miner tool.
-It does not import any local default site packages to decrease the attack surface due to manipulation of unused but available packages.
+"""This is the main program of the "aminer" logfile miner tool. It does not
+import any local default site packages to decrease the attack surface due to
+manipulation of unused but available packages.
 
 CAVEAT: This process will keep running with current permissions, no matter what was specified in 'AminerUser' and 'AminerGroup'
-configuration properties. This is required to allow the aminer parent parent process to reopen log files, which might need the
+configuration properties. This is required to allow the aminer parent process to reopen log files, which might need the
 elevated privileges.
 
 NOTE: This tool is developed to allow secure operation even in hostile environment, e.g. when one directory, where aminer attempts
-to open logfiles is already under full control of an attacker. However it is not intended to be run as SUID-binary, this would
+to open logfiles is already under full control of an attacker. However, it is not intended to be run as SUID-binary, this would
 require code changes to protect also against standard SUID attacks.
 
 Parameters:
@@ -43,14 +42,18 @@ from grp import getgrnam
 from logging.handlers import RotatingFileHandler
 
 # As site packages are not included, define from where we need to execute code before loading it.
-sys.path = sys.path[1:] + ['/usr/lib/logdata-anomaly-miner', '/etc/aminer/conf-enabled']
-import aminer.AminerConfig as AminerConfig  # skipcq: FLK-E402
-from aminer.util.StringUtil import colflame, flame, supports_color, decode_string_as_byte_string  # skipcq: FLK-E402
-from aminer.util.PersistenceUtil import clear_persistence, copytree  # skipcq: FLK-E402
-from aminer.util import SecureOSFunctions  # skipcq: FLK-E402
-from aminer.AnalysisChild import AnalysisChild  # skipcq: FLK-E402
-from aminer.input.LogStream import FileLogDataResource, UnixSocketLogDataResource  # skipcq: FLK-E402
-from metadata import __version_string__, __version__  # skipcq: FLK-E402
+sys.path = sys.path[1:] + ["/usr/lib/logdata-anomaly-miner", "/etc/aminer/conf-enabled"]
+venv_path = "/usr/lib/logdata-anomaly-miner/.venv/lib"
+if os.path.exists(venv_path):
+    python_version = os.listdir(venv_path)[0]
+    sys.path += [os.path.join(venv_path, python_version, "site-packages")]
+import aminer.AminerConfig as AminerConfig  # noqa: E402
+from aminer.util.StringUtil import colflame, flame, supports_color, decode_string_as_byte_string  # noqa: E402
+from aminer.util.PersistenceUtil import clear_persistence, copytree  # noqa: E402
+from aminer.util import SecureOSFunctions  # noqa: E402
+from aminer.AnalysisChild import AnalysisChild  # noqa: E402
+from aminer.input.LogStream import FileLogDataResource, UnixSocketLogDataResource  # noqa: E402
+from metadata import __version_string__, __version__  # noqa: E402
 
 
 child_termination_triggered_flag = False
@@ -73,7 +76,7 @@ def run_analysis_child(aminer_config, program_name):
         logging.getLogger(AminerConfig.DEBUG_LOG_NAME).critical(msg)
         sys.exit(1)
     import posix1e
-    # O_PATH is problematic when checking ACL. However it is possible to check the ACL using the file name.
+    # O_PATH is problematic when checking ACL. However, it is possible to check the ACL using the file name.
     try:
         if posix1e.has_extended(persistence_dir_name):
             msg = f"WARNING: SECURITY: Extended POSIX ACLs are set in {persistence_dir_name.decode()}, but not supported by the aminer. " \
@@ -191,14 +194,11 @@ def initialize_loggers(aminer_config, aminer_user_id, aminer_grp_id):
 
 
 def parse_var(s):
-    """
-    Parse a key, value pair, separated by "=".
-    That's the reverse of ShellArgs.
+    """Parse a key, value pair, separated by "=". That's the reverse of
+    ShellArgs.
 
-    On the command line (argparse) a declaration will typically look like:
-        foo=hello
-    or
-        foo="hello world"
+    On the command line (argparse) a declaration will typically look
+    like:     foo=hello or     foo="hello world"
     """
     items = s.split("=")
     key = items[0].strip()  # we remove blanks around keys, as is logical
@@ -289,7 +289,7 @@ def main():
     clear_persistence_flag = args.clear
     remove_persistence_dirs = args.remove
     from_begin_flag = args.from_begin
-    global offline_mode  # skipcq: PYL-W0603
+    global offline_mode
     offline_mode = args.offline_mode
     if args.restore is not None and ('.' in args.restore or '/' in args.restore):
         parser.error(f"The restore path {args.restore} must not contain any . or /")
@@ -390,7 +390,7 @@ def main():
             child_user_id = getpwnam(child_user_name).pw_uid
         if child_group_name is not None:
             child_group_id = getgrnam(child_group_name).gr_gid
-    except:  # skipcq: FLK-E722
+    except KeyError:
         print(f"Failed to resolve {AminerConfig.KEY_AMINER_USER} or {AminerConfig.KEY_AMINER_GROUP}", file=sys.stderr)
         sys.exit(1)
 
@@ -489,38 +489,76 @@ def main():
 
     # Now create the management entries for each logfile.
     log_data_resource_dict = {}
-    for log_resource_name in log_sources_list:
-        # From here on log_resource_name is a byte array.
-        log_resource_name = decode_string_as_byte_string(log_resource_name)
-        log_resource = None
-        if log_resource_name.startswith(b'file://'):
-            log_resource = FileLogDataResource(log_resource_name, -1)
-        elif log_resource_name.startswith(b'unix://'):
-            log_resource = UnixSocketLogDataResource(log_resource_name, -1)
+    for resource in log_sources_list:
+        obj = {}
+        if isinstance(resource, str):
+            obj["url"] = decode_string_as_byte_string(resource)
+        elif isinstance(resource, dict):
+            for key, val in resource.items():
+                if key not in ("url", "json", "xml", "parser_id"):
+                    msg = f"Unknown argument in LogResourceList: {key}"
+                    print(msg, file=sys.stderr)
+                    logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
+                    sys.exit(1)
+                if key == "json" and not isinstance(val, bool):
+                    msg = "Argument json must be of type boolean!"
+                    print(msg, file=sys.stderr)
+                    logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
+                    sys.exit(1)
+                if key == "xml" and not isinstance(val, bool):
+                    msg = "Argument xml must be of type boolean!"
+                    print(msg, file=sys.stderr)
+                    logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
+                    sys.exit(1)
+                obj[key] = val
         else:
-            msg = f"Unsupported schema in {AminerConfig.KEY_LOG_SOURCES_LIST}: {repr(log_resource_name)}"
+            msg = "LogResourceList must be of type dict or string"
             print(msg, file=sys.stderr)
             logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
             sys.exit(1)
-        if not os.path.exists(log_resource_name[7:].decode()):
-            msg = f"WARNING: file or socket '{log_resource_name[7:].decode()}' does not exist (yet)!"
+        if "json" in obj and "xml" in obj:
+            msg = "Log resources can not be in the json and xml format at the same time."
+            logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
+            raise ValueError(msg)
+        if "json" not in obj:
+            obj["json"] = None
+        if "xml" not in obj:
+            obj["xml"] = None
+        if "parser_id" not in obj:
+            obj["parser_id"] = None
+        if isinstance(obj["url"], str):
+            obj["url"] = decode_string_as_byte_string(obj["url"])
+        url = obj["url"]
+        if url.startswith(b'file://'):
+            obj["log_resource"] = FileLogDataResource(url, -1)
+        elif url.startswith(b'unix://'):
+            obj["log_resource"] = UnixSocketLogDataResource(url, -1)
+        else:
+            msg = "Config-Error: {'LogResourceList': 'Every log resource URL must be prefixed with either file:// or unix://.'}"
+            print(msg, file=sys.stderr)
+            logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
+            sys.exit(1)
+        if obj["xml"]:
+            obj["log_resource"].default_buffer_size = 1 << 32
+        if not os.path.exists(url[7:].decode()):
+            msg = f"WARNING: file or socket '{url[7:].decode()}' does not exist (yet)!"
             print(msg, file=sys.stderr)
             logging.getLogger(AminerConfig.DEBUG_LOG_NAME).warning(msg)
         try:
-            log_resource.open()
+            obj["log_resource"].open()
         except OSError as open_os_error:
             if open_os_error.errno == errno.EACCES:
-                msg = f"{program_name}: no permission to access{repr(log_resource_name)}"
+                msg = f"{program_name}: no permission to access{repr(obj)}"
                 print(msg, file=sys.stderr)
                 logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
                 sys.exit(1)
             else:
-                msg = f"{program_name}: unexpected error opening {repr(log_resource_name)}: {open_os_error.errno} " \
+                msg = f"{program_name}: unexpected error opening {repr(obj)}: {open_os_error.errno} " \
                       f"({os.strerror(open_os_error.errno)})"
                 print(msg, file=sys.stderr)
                 logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
                 sys.exit(1)
-        log_data_resource_dict[log_resource_name] = log_resource
+        log_data_resource_dict[url] = obj
 
     # Create the remote control socket, if any. Do this in privileged mode to allow binding it at arbitrary locations and support restricted
     # permissions of any type for current (privileged) uid.
@@ -544,7 +582,7 @@ def main():
             os.umask(0o177)
             remote_control_socket.bind(remote_control_socket_name)
             # Do not perform any cleanup, flushing of streams. Use _exit(0) to avoid interference with fork.
-            os._exit(0)  # skipcq: PYL-W0212
+            os._exit(0)
         os.waitpid(bind_child_pid, 0)
         remote_control_socket.listen(4)
 
@@ -554,34 +592,33 @@ def main():
         try:
             # Fork a child to make sure, we are not the process group leader already.
             child_pid = os.fork()
-        except Exception as fork_exception:  # skipcq: PYL-W0703
+        except Exception as fork_exception:
             msg = 'Failed to daemonize: %s' % fork_exception
             print(msg, file=sys.stderr)
             logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
             sys.exit(1)
         if child_pid != 0:
             # This is the parent.
-            os._exit(0)  # skipcq: PYL-W0212
+            os._exit(0)
         # This is the child. Create a new session and become process group leader. Here we get rid of the controlling tty.
         os.setsid()
         # Fork again to become an orphaned process not being session leader, hence not able to get a controlling tty again.
         try:
             child_pid = os.fork()
-        except Exception as fork_exception:  # skipcq: PYL-W0703
+        except Exception as fork_exception:
             msg = f"Failed to daemonize: {fork_exception}"
             print(msg, file=sys.stderr)
             logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
             sys.exit(1)
         if child_pid != 0:
             # This is the parent.
-            os._exit(0)  # skipcq: PYL-W0212
+            os._exit(0)
         # Move to root directory to avoid lingering in some cwd someone else might want to unmount.
         os.chdir('/')
-        # Change the umask here to clean all group/other mask bits so that accidentially created files are not accessible by other.
+        # Change the umask here to clean all group/other mask bits so that accidentially created files are not accessible by others.
         os.umask(0o77)
 
     # Install a signal handler catching common stop signals and relaying it to all children for sure.
-    # skipcq: PYL-W0603
     global child_termination_triggered_flag
     child_termination_triggered_flag = False
 
@@ -592,7 +629,6 @@ def main():
         logging.getLogger(AminerConfig.DEBUG_LOG_NAME).info(msg)
         # Just set the flag. It is likely, that child received same signal also so avoid multiple signaling, which could interrupt the
         # shutdown procedure again.
-        # skipcq: PYL-W0603
         global child_termination_triggered_flag
         child_termination_triggered_flag = True
 
@@ -639,7 +675,7 @@ def main():
                     aminer_user_id = getpwnam(tmp_username).pw_uid
                 if tmp_group is not None:
                     aminer_group_id = getgrnam(tmp_group).gr_gid
-            except:  # skipcq: FLK-E722
+            except KeyError:
                 print(f"Failed to resolve {AminerConfig.KEY_AMINER_USER} or {AminerConfig.KEY_AMINER_GROUP}", file=sys.stderr)
                 sys.exit(1)
 
@@ -665,7 +701,7 @@ def main():
                 logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
                 # Flush stderr before exit without any cleanup.
                 sys.stderr.flush()
-                os._exit(1)  # skipcq: PYL-W0212
+                os._exit(1)
 
         # Now execute the very same program again, but user might have moved or renamed it meanwhile. This would be problematic with
         # SUID-binaries (which we do not yet support). Do NOT just fork but also exec to avoid child circumventing
@@ -678,18 +714,19 @@ def main():
             exec_args.append("--config-properties")
             for config_property in args.config_properties:
                 exec_args.append(config_property)
-        os.execv(sys.argv[0], exec_args)  # skipcq: BAN-B606
+        os.execv(sys.argv[0], exec_args)  # nosec B606
         msg = 'Failed to execute child process'
         print(msg, file=sys.stderr)
         logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
         sys.stderr.flush()
-        os._exit(1)  # skipcq: PYL-W0212
+        os._exit(1)
     child_socket.close()
 
     # Send all log resource information currently available to child process.
-    for log_resource_name, log_resource in log_data_resource_dict.items():
+    for url, obj in log_data_resource_dict.items():
+        log_resource = obj["log_resource"]
         if (log_resource is not None) and (log_resource.get_file_descriptor() >= 0):
-            SecureOSFunctions.send_logstream_descriptor(parent_socket, log_resource.get_file_descriptor(), log_resource_name)
+            SecureOSFunctions.send_logstream_descriptor(parent_socket, log_resource.get_file_descriptor(), url)
             log_resource.close()
 
     # Send the remote control server socket, if any and close it afterwards. It is not needed any more on parent side.
@@ -726,9 +763,10 @@ def main():
             exit_status = 1
 
         # Child information handled, scan for rotated logfiles or other resources, where reopening might make sense.
-        for log_resouce_name, log_data_resource in log_data_resource_dict.items():
+        for log_resouce_name, obj in log_data_resource_dict.items():
+            log_resource = obj["log_resource"]
             try:
-                if not log_data_resource.open(reopen_flag=True):
+                if not log_resource.open(reopen_flag=True):
                     continue
             except OSError as open_os_error:
                 if open_os_error.errno == errno.EACCES:
@@ -742,8 +780,8 @@ def main():
                     logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
                 exit_status = 2
                 continue
-            SecureOSFunctions.send_logstream_descriptor(parent_socket, log_data_resource.get_file_descriptor(), log_resouce_name)
-            log_data_resource.close()
+            SecureOSFunctions.send_logstream_descriptor(parent_socket, log_resource.get_file_descriptor(), log_resouce_name)
+            log_resource.close()
         time.sleep(1)
     parent_socket.close()
     SecureOSFunctions.close_base_directory()
