@@ -1,9 +1,8 @@
-"""
-This module defines a PCA-detector for event and value counts.
-The component detects anomalies by creating an Event-Count-Matrix for given
-time-windows to calculate an anomaly score for new time windows afterwards by
-using the reconstruction error from the inverse-transformation with restricted
-components of the Principal-Component-Analysis (PCA).
+"""This module defines a PCA-detector for event and value counts. The component
+detects anomalies by creating an Event-Count-Matrix for given time-windows to
+calculate an anomaly score for new time windows afterwards by using the
+reconstruction error from the inverse-transformation with restricted components
+of the Principal-Component-Analysis (PCA).
 
 This program is free software: you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -19,26 +18,27 @@ import copy
 import numpy as np
 import logging
 import os
-import time
 from aminer import AminerConfig
-from aminer.AminerConfig import DEBUG_LOG_NAME, STAT_LEVEL, STAT_LOG_NAME, CONFIG_KEY_LOG_LINE_PREFIX, DEFAULT_LOG_LINE_PREFIX,\
+from aminer.AminerConfig import DEBUG_LOG_NAME, STAT_LEVEL, STAT_LOG_NAME, CONFIG_KEY_LOG_LINE_PREFIX, DEFAULT_LOG_LINE_PREFIX, \
     KEY_PERSISTENCE_PERIOD, DEFAULT_PERSISTENCE_PERIOD
 from aminer.AnalysisChild import AnalysisContext
 from aminer.util import PersistenceUtil
-from aminer.input.InputInterfaces import AtomHandlerInterface
+from aminer.input.InputInterfaces import AtomHandlerInterface, PersistableComponentInterface
 from aminer.util.TimeTriggeredComponentInterface import TimeTriggeredComponentInterface
 
 
-class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
-    """This class creates events if event or value occurrence counts are outliers in PCA space."""
+class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface, PersistableComponentInterface):
+    """This class creates events if event or value occurrence counts are
+    outliers in PCA space."""
 
     time_trigger_class = AnalysisContext.TIME_TRIGGER_CLASS_REALTIME
 
     def __init__(self, aminer_config, target_path_list, anomaly_event_handlers, window_size, min_anomaly_score, min_variance, num_windows,
-                 persistence_id='Default', learn_mode=False, output_logline=True, ignore_list=None, constraint_list=None,
-                 stop_learning_time=None, stop_learning_no_anomaly_time=None):
-        """
-        Initialize the detector. This will also trigger reading or creation of persistence storage location.
+                 persistence_id="Default", learn_mode=False, output_logline=True, ignore_list=None, constraint_list=None,
+                 stop_learning_time=None, stop_learning_no_anomaly_time=None, log_resource_ignore_list=None):
+        """Initialize the detector. This will also trigger reading or creation
+        of persistence storage location.
+
         @param aminer_config configuration from analysis_context.
         @param target_path_list parser paths of values to be analyzed. Multiple paths mean that values are analyzed as separate
                dimensions. When no paths are specified, the events given by the full path list are analyzed (one dimension).
@@ -58,21 +58,24 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
         @param stop_learning_no_anomaly_time switch the learn_mode to False after no anomaly was detected for that time.
         """
         # avoid "defined outside init" issue
-        self.learn_mode, self.stop_learning_timestamp, self.next_persist_time, self.log_success, self.log_total = [None]*5
+        self.learn_mode, self.stop_learning_time, self.next_persist_time, self.log_success, self.log_total = [None]*5
+        self.stop_learning_time_initialized = None
         super().__init__(
-            mutable_default_args=["ignore_list", "constraint_list"], aminer_config=aminer_config, target_path_list=target_path_list,
-            anomaly_event_handlers=anomaly_event_handlers, window_size=window_size, min_anomaly_score=min_anomaly_score,
-            min_variance=min_variance, num_windows=num_windows, persistence_id=persistence_id, learn_mode=learn_mode,
-            output_logline=output_logline, ignore_list=ignore_list, constraint_list=constraint_list, stop_learning_time=stop_learning_time,
-            stop_learning_no_anomaly_time=stop_learning_no_anomaly_time
+            mutable_default_args=["ignore_list", "constraint_list", "log_resource_ignore_list"], aminer_config=aminer_config,
+            target_path_list=target_path_list, anomaly_event_handlers=anomaly_event_handlers, window_size=window_size,
+            min_anomaly_score=min_anomaly_score, min_variance=min_variance, num_windows=num_windows, persistence_id=persistence_id,
+            learn_mode=learn_mode, output_logline=output_logline, ignore_list=ignore_list, constraint_list=constraint_list,
+            stop_learning_time=stop_learning_time, stop_learning_no_anomaly_time=stop_learning_no_anomaly_time,
+            log_resource_ignore_list=log_resource_ignore_list
         )
-        # skipcq: PYL-W0511
-        # ToDo: an exception should be thrown instead of this check.
+        if not self.target_path_list:
+            msg = "target_path_list must not be empty or None."
+            logging.getLogger(DEBUG_LOG_NAME).error(msg)
+            raise ValueError(msg)
         if num_windows < 3:
-            logging.getLogger(AminerConfig.DEBUG_LOG_NAME).warning('num_windows must be >= 3!')
-            self.num_windows = 3
-        else:
-            self.num_windows = num_windows
+            msg = "num_windows must be greater than 2."
+            logging.getLogger(DEBUG_LOG_NAME).error(msg)
+            raise ValueError(msg)
         self.first_log = True
         self.start_time = 0
         self.event_count_matrix = []
@@ -85,37 +88,34 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
         self.n_comp = None
         # Calculate Anomaly-Score (Reconstruction Error) for the whole dataset
         self.loss = None
+        self.event_count_vector = {}
 
         self.persistence_file_name = AminerConfig.build_persistence_file_name(aminer_config, self.__class__.__name__, persistence_id)
         PersistenceUtil.add_persistable_component(self)
-        persistence_data = PersistenceUtil.load_json(self.persistence_file_name)
-
-        if persistence_data is not None:
-            self.event_count_matrix = list(persistence_data)
-            self.compute_pca()
-            # Copy feature list into event count vector and reset counts of each feature
-            self.event_count_vector = copy.deepcopy(self.event_count_matrix[0])
-            self.reset_event_count_vector()
-        else:
-            if self.target_path_list is None or len(self.target_path_list) == 0:
-                # Only one dimension when events are used instead of values; use empty string as placeholder
-                self.event_count_vector = {'': {}}
-            else:
-                self.event_count_vector = {}
+        self.load_persistence_data()
 
     def receive_atom(self, log_atom):
         """Receive parsed atom and the information about the parser match."""
+        for source in self.log_resource_ignore_list:
+            if log_atom.source.resource_name.decode() == source:
+                return False
+        if not self.stop_learning_time_initialized:
+            self.stop_learning_time_initialized = True
+            if self.stop_learning_time is not None:
+                self.stop_learning_time = log_atom.atom_time + self.stop_learning_time
+            elif self.stop_learning_no_anomaly_time is not None:
+                self.stop_learning_time = log_atom.atom_time + self.stop_learning_no_anomaly_time
+
         parser_match = log_atom.parser_match
         self.log_total += 1
-        if self.learn_mode is True and self.stop_learning_timestamp is not None and \
-                self.stop_learning_timestamp < log_atom.atom_time:
+        if self.learn_mode is True and self.stop_learning_time is not None and self.stop_learning_time < log_atom.atom_time:
             logging.getLogger(DEBUG_LOG_NAME).info("Stopping learning in the %s.", self.__class__.__name__)
             self.learn_mode = False
 
         # Skip paths from ignore list.
         for ignore_path in self.ignore_list:
             if ignore_path in parser_match.get_match_dictionary().keys():
-                return
+                return False
 
         # get the timestamp of the first log to start the time-window-process (flag)
         if self.first_log:
@@ -125,7 +125,7 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
         current_time = log_atom.get_timestamp()
         while current_time >= (self.start_time + self.window_size):
             # PCA computation only possible when at least 3 vectors are present
-            if len(self.event_count_matrix) >= 3:
+            if len(self.event_count_matrix) >= 3 and all(all(len(x.values()) >= 2 for x in y.values()) for y in self.event_count_matrix):
                 anomaly_score = self.anomaly_score()
                 if anomaly_score > self.min_anomaly_score:
                     try:
@@ -135,7 +135,7 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
                     if self.output_logline:
                         original_log_line_prefix = self.aminer_config.config_properties.get(
                             CONFIG_KEY_LOG_LINE_PREFIX, DEFAULT_LOG_LINE_PREFIX)
-                        sorted_log_lines = [log_atom.parser_match.match_element.annotate_match('') + os.linesep + original_log_line_prefix +
+                        sorted_log_lines = [log_atom.parser_match.match_element.annotate_match("") + os.linesep + original_log_line_prefix +
                                             data]
                     else:
                         sorted_log_lines = [data]
@@ -146,11 +146,11 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
                         affected_paths.append(path)
                         affected_values.append(list(count_dict.keys()))
                         affected_counts.append(list(count_dict.values()))
-                    analysis_component = {'AffectedLogAtomPaths': affected_paths, 'AffectedLogAtomValues': affected_values,
-                                          'AffectedValueCounts': affected_counts, 'AnomalyScore': anomaly_score[0]}
-                    event_data = {'AnalysisComponent': analysis_component}
+                    analysis_component = {"AffectedLogAtomPaths": affected_paths, "AffectedLogAtomValues": affected_values,
+                                          "AffectedValueCounts": affected_counts, "AnomalyScore": anomaly_score[0]}
+                    event_data = {"AnalysisComponent": analysis_component}
                     for listener in self.anomaly_event_handlers:
-                        listener.receive_event(f'Analysis.{self.__class__.__name__}', 'PCA anomaly detected', sorted_log_lines, event_data,
+                        listener.receive_event(f"Analysis.{self.__class__.__name__}", "PCA anomaly detected", sorted_log_lines, event_data,
                                                log_atom, self)
             self.log_windows += 1
 
@@ -160,11 +160,11 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
                     del self.event_count_matrix[0]
                 self.event_count_matrix.append(copy.deepcopy(self.event_count_vector))
                 # PCA computation only possible when at least 3 vectors are present
-                if len(self.event_count_matrix) >= 3:
+                if len(self.event_count_matrix) >= 3 and all(all(len(x.values()) > 1 for x in y.values()) for y in self.event_count_matrix):
                     self.repair_dict()
                     self.compute_pca()
-                if self.stop_learning_timestamp is not None and self.stop_learning_no_anomaly_time is not None:
-                    self.stop_learning_timestamp = time.time() + self.stop_learning_no_anomaly_time
+                if self.stop_learning_time is not None and self.stop_learning_no_anomaly_time is not None:
+                    self.stop_learning_time = max(self.stop_learning_time, log_atom.atom_time + self.stop_learning_no_anomaly_time)
 
             # Set window end time for next iteration
             self.start_time += self.window_size
@@ -179,12 +179,12 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
                     constraint_path_flag = True
                     break
             if not constraint_path_flag and self.constraint_list != []:
-                return
+                return False
             log_event = tuple(parser_match.get_match_dictionary().keys())
-            if log_event in self.event_count_vector['']:
-                self.event_count_vector[''][log_event] += 1
+            if log_event in self.event_count_vector[""]:
+                self.event_count_vector[""][log_event] += 1
             else:
-                self.event_count_vector[''][log_event] = 1
+                self.event_count_vector[""][log_event] = 1
         else:
             # Event is defined by values in target_path_list
             all_values_none = True
@@ -212,8 +212,9 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
                     else:
                         self.event_count_vector[path] = {value: 1}
             if all_values_none is True:
-                return
+                return False
         self.log_success += 1
+        return True
 
     def compute_pca(self):
         """Carry out PCA on current event count matrix."""
@@ -267,16 +268,18 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
         return loss
 
     def vector2array(self):
-        """Extract only the values which were learned before from current self.event_count_vector and return an array."""
+        """Extract only the values which were learned before from current
+        self.event_count_vector and return an array."""
         vector = []
         for event in self.event_count_vector.values():
-            for feature, value in event.items():
-                if feature in self.feature_list:
-                    vector.append(value)
+            for key in self.feature_list:
+                if key in event.keys():
+                    vector.append(event[key])
         return np.array(vector)
 
     def get_n_comp(self, eigen_values):
-        """Return the number of components, which describe the variance threshold."""
+        """Return the number of components, which describe the variance
+        threshold."""
         # Calculate the explained variance on each of components
         variance_explained = []
         for i in eigen_values[::-1]:
@@ -289,7 +292,8 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
         return None
 
     def repair_dict(self):
-        """Check if any new values were added in current event_count_vector and repair self.event_count_matrix when necessary."""
+        """Check if any new values were added in current event_count_vector and
+        repair self.event_count_matrix when necessary."""
         for ecv in self.event_count_matrix:
             for key, value in self.event_count_vector.items():
                 if key not in ecv.keys():
@@ -315,53 +319,69 @@ class PCADetector(AtomHandlerInterface, TimeTriggeredComponentInterface):
         if delta <= 0:
             self.do_persist()
             delta = self.aminer_config.config_properties.get(KEY_PERSISTENCE_PERIOD, DEFAULT_PERSISTENCE_PERIOD)
-            self.next_persist_time = time.time() + delta
+            self.next_persist_time = trigger_time + delta
         return delta
 
     def do_persist(self):
         """Immediately write persistence data to storage."""
-        if self.learn_mode is True:
-            PersistenceUtil.store_json(self.persistence_file_name, list(self.event_count_matrix))
+        PersistenceUtil.store_json(self.persistence_file_name, list(self.event_count_matrix))
+
+    def load_persistence_data(self):
+        """Load the persistence data from storage."""
+        persistence_data = PersistenceUtil.load_json(self.persistence_file_name)
+        if persistence_data is not None:
+            self.event_count_matrix = list(persistence_data)
+            self.compute_pca()
+            # Copy feature list into event count vector and reset counts of each feature
+            self.event_count_vector = copy.deepcopy(self.event_count_matrix[0])
+            self.reset_event_count_vector()
+        else:
+            if self.target_path_list is None or len(self.target_path_list) == 0:
+                # Only one dimension when events are used instead of values; use empty string as placeholder
+                self.event_count_vector = {"": {}}
 
     def allowlist_event(self, event_type, event_data, allowlisting_data):
-        """
-        Allowlist an event generated by this source using the information emitted when generating the event.
+        """Allowlist an event generated by this source using the information
+        emitted when generating the event.
+
         @return a message with information about allowlisting
         @throws Exception when allowlisting of this special event using given allowlisting_data was not possible.
         """
-        if event_type != f'Analysis.{self.__class__.__name__}':
-            msg = 'Event not from this source'
+        if event_type != f"Analysis.{self.__class__.__name__}":
+            msg = "Event not from this source"
             logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
             raise Exception(msg)
         if allowlisting_data is not None:
-            msg = 'Allowlisting data not understood by this detector'
+            msg = "Allowlisting data not understood by this detector"
             logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
             raise Exception(msg)
         if event_data not in self.constraint_list:
             self.constraint_list.append(event_data)
-        return f'Allowlisted path {event_data}.'
+        return f"Allowlisted path {event_data} in {event_type}."
 
     def blocklist_event(self, event_type, event_data, blocklisting_data):
-        """
-        Blocklist an event generated by this source using the information emitted when generating the event.
+        """Blocklist an event generated by this source using the information
+        emitted when generating the event.
+
         @return a message with information about blocklisting
         @throws Exception when blocklisting of this special event using given blocklisting_data was not possible.
         """
-        if event_type != f'Analysis.{self.__class__.__name__}':
-            msg = 'Event not from this source'
+        if event_type != f"Analysis.{self.__class__.__name__}":
+            msg = "Event not from this source"
             logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
             raise Exception(msg)
         if blocklisting_data is not None:
-            msg = 'Blocklisting data not understood by this detector'
+            msg = "Blocklisting data not understood by this detector"
             logging.getLogger(AminerConfig.DEBUG_LOG_NAME).error(msg)
             raise Exception(msg)
         if event_data not in self.ignore_list:
             self.ignore_list.append(event_data)
-        return f'Blocklisted path {event_data}.'
+        return f"Blocklisted path {event_data} in {event_type}."
 
     def log_statistics(self, component_name):
-        """
-        Log statistics of an AtomHandler. Override this method for more sophisticated statistics output of the AtomHandler.
+        """Log statistics of an AtomHandler.
+
+        Override this method for more sophisticated statistics output of the AtomHandler.
         @param component_name the name of the component which is printed in the log line.
         """
         if STAT_LEVEL == 1:
