@@ -1,5 +1,4 @@
-"""
-This module defines a model element that matches any byte.
+"""This module defines a model element that matches any byte.
 
 This program is free software: you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -16,11 +15,15 @@ from aminer.parsing.ModelElementInterface import ModelElementInterface
 
 
 class AnyByteDataModelElement(ModelElementInterface):
-    """This class matches any byte but at least one. Thus, a match will always span the complete data from beginning to end."""
+    """This class matches any byte but at least one.
+
+    Thus, a match will always span the complete data from beginning to
+    end.
+    """
 
     def get_match_element(self, path: str, match_context):
-        """
-        Just return a match including all data from the context.
+        """Just return a match including all data from the context.
+
         @param path to be printed in the MatchElement.
         @param match_context the match_context to be analyzed.
         """
