@@ -1,5 +1,4 @@
-"""
-Some useful string-functions.
+"""Some useful string-functions.
 
 This program is free software: you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -40,8 +39,9 @@ flame = ("           *     (       )       (     \n"
 
 
 def supports_color():
-    """
-    Return True if the running system's terminal supports color, and False otherwise.
+    """Return True if the running system's terminal supports color, and False
+    otherwise.
+
     The function was borrowed from the django-project (https://github.com/django/django/blob/master/django/core/management/color.py)
     """
     plat = sys.platform
@@ -52,8 +52,9 @@ def supports_color():
 
 
 def decode_string_as_byte_string(string):
-    """
-    Decode a string produced by the encode function encodeByteStringAsString(byteString) below.
+    """Decode a string produced by the encode function
+    encodeByteStringAsString(byteString) below.
+
     @return string.
     """
     decoded = b''
@@ -73,9 +74,10 @@ def decode_string_as_byte_string(string):
 
 
 def encode_byte_string_as_string(byte_string):
-    r"""
-    Encode an arbitrary byte string to a string.
-    This is achieved by replacing all non ascii-7 bytes and all non printable ascii-7 bytes and % character by replacing with their escape
+    r"""Encode an arbitrary byte string to a string. This is achieved by
+    replacing all non ascii-7 bytes and all non printable ascii-7 bytes and %
+    character by replacing with their escape.
+
     sequence %[hex]. For example byte string b'/\xc3' is encoded to '/%c3'
     @return a string with decoded name.
     """
@@ -84,5 +86,5 @@ def encode_byte_string_as_string(byte_string):
         if byte in b'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890!"#$&\'()*+,-./:;<=>?@[]\\^_`{}|~ ':
             encoded += chr(byte)
         else:
-            encoded += '%%%02x' % byte  # skipcq: PYL-C0209
+            encoded += '%%%02x' % byte
     return encoded
